#!/usr/bin/env Rscript

## Initialization ####
figPath <- root_folder
aux <- str_split(folder, pattern = "/")[[1]]
aux <- aux[length(aux)]

# Get all files in path
files <- list.files(path = dPath, full.names = TRUE, pattern = ".xlsx$")

# Get list of filenames from list of files
fnames = basename(files)
fnames = gsub(".xlsx", "", fnames)
scenario_names = gsub("URBS_", "", fnames)

# General function to add columns with model names and scenario names for one list of observations:
fnames_split <- fnames %>% str_split_fixed(pattern = "_", n = 2) 
colnames(fnames_split) = c("Model", "Scenario")
add_type <- function(x) {
  for (i in 1:length(x)) {
    x[[i]]$Model <- fnames_split[[i, 1]]
    x[[i]]$Scenario <- fnames_split[[i, 2]]
  }
  return(x)
}
model_names <- unique(fnames_split[, 1])

# Capitalize first letter
firstup <- function(x) {
  substr(x, 1, 1) <- toupper(substr(x, 1, 1))
  x
}

## Load data in loop ####
mysheetnames = list()
i = 1

for(s in mysheets[-13]){
  mystuff = lapply(files, function(x) data.frame(read_excel(x, sheet = paste0(s), na = c("","Eps"))))
  s = gsub("-", "_", s)
  s = gsub(" ", "_", s)
  if (s %in% clean_filter){
    mystuff <- mystuff %>% lapply(subset, Site %in% region_names)
    mystuff <- lapply(mystuff, function(x) replace(x,is.na(x),0))
  }
  mystuff <- add_type(mystuff)
  assign(paste0("df_", s), mystuff) 
  mysheetnames[[i]] <- paste0("df_", s)
  i = i+1
}
rm(mystuff)
mysheetnames = unlist(mysheetnames)

## Electricity trade ####

df <- df_Transfers
df_all <- do.call("rbind", df)

electricity_transfers <- df_all %>% select(scenario.year, Model, Scenario, Site, region_names)
electricity_transfers <- electricity_transfers %>% filter(Site %in% region_names)
electricity_transfers_T <- electricity_transfers %>% 
  gather(key, value, region_names) %>%
  spread(Site, value)
electricity_transfers <- electricity_transfers %>% gather(region_names, key = "Type", value = "MW")
electricity_transfers[is.na(electricity_transfers)] <- 0
electricity_transfers$MW <- - as.numeric(electricity_transfers$MW)
electricity_transfers$TWh <- electricity_transfers$MW / 10^6
electricity_transfers$Type <- firstup(electricity_transfers$Type)
electricity_transfers_T <- electricity_transfers_T %>% gather(region_names, key = "Type", value = "MW")
electricity_transfers_T$Site <- electricity_transfers_T$key
electricity_transfers_T$key <- NULL
electricity_transfers_T[is.na(electricity_transfers_T)] <- 0
electricity_transfers_T$MW <- as.numeric(electricity_transfers_T$MW)
electricity_transfers_T$TWh <- electricity_transfers_T$MW / 10^6
electricity_transfers_T$Type <- firstup(electricity_transfers_T$Type)
electricity_transfers <- electricity_transfers[order(electricity_transfers$scenario.year,
                                                     electricity_transfers$Model,
                                                     electricity_transfers$Scenario,
                                                     electricity_transfers$Type,
                                                     electricity_transfers$Site),]
electricity_transfers_T <- electricity_transfers_T[order(electricity_transfers_T$scenario.year,
                                                         electricity_transfers_T$Model,
                                                         electricity_transfers_T$Scenario,
                                                         electricity_transfers_T$Type,
                                                         electricity_transfers_T$Site),]
electricity_transfers$MW <- electricity_transfers$MW + electricity_transfers_T$MW
electricity_transfers$TWh <- electricity_transfers$TWh + electricity_transfers_T$TWh

electricity_transfers_simple <- electricity_transfers %>%
  group_by(scenario.year, Model, Scenario, Site) %>%
  summarise(MW = sum(MW),
            TWh = sum(TWh))
electricity_transfers_simple$Type <- "Imports - Exports"

## Fig 2 ####

if (aux=="focus on energy"){
  
  df <- df_Electricity_generation
  df_all <- do.call("bind_rows", df)
  df_all[is.na(df_all)] <- 0
  df_all <- cbind(select(df_all, !starts_with("Hydro")),
                  transmute(df_all, Hydro = rowSums(select(df_all, starts_with("Hydro")))))
  df_all <- cbind(select(df_all, !starts_with("Solar")),
                  transmute(df_all, Solar = rowSums(select(df_all, starts_with("Solar")))))
  df_all <- cbind(select(df_all, !starts_with("WindOn")),
                  transmute(df_all, WindOn = rowSums(select(df_all, starts_with("WindOn")))))
  
  
  df <- df_all %>%
    group_by(scenario.year, Model, Scenario, Site) %>%
    summarise(Bioenergy = sum(Bioenergy),
              Coal = sum(Coal),
              Gas = sum(Gas),
              Hydro = sum(Hydro),
              Import = sum(Import),
              Waste_heat = sum(Waste_heat),
              Oil = sum(Oil), 
              Solar = sum(Solar),
              WindOn = sum(WindOn))
  
  electricity_generation_regional <- df %>% gather(gather_vector[-5], key = "Type", value = "MW")
  electricity_generation_regional$TWh <- electricity_generation_regional$MW / 10^6
  electricity_generation_regional$Site <- firstup(electricity_generation_regional$Site)
  electricity_generation_regional$Type <- electricity_generation_regional$Type %>% factor(levels = legend_order)
  
  electricity_generation_regional$Import <- NULL
  electricity_transfers_simple <- rbind(electricity_transfers_simple, electricity_generation_regional)
  electricity_transfers_simple$Site <- firstup(electricity_transfers_simple$Site)
  electricity_transfers_simple$Type <- recode(electricity_transfers_simple$Type %>% factor(levels = c("Imports - Exports", legend_order)),
                                                 "WindOn" = "Onshore wind",
                                                 "Waste_heat" = "Waste heat")
  electricity_generation_regional <- rbind(electricity_transfers, electricity_generation_regional)
  electricity_generation_regional$Site <- firstup(electricity_generation_regional$Site)
  electricity_generation_regional$Type <- recode(electricity_generation_regional$Type %>% factor(levels = c(legend_trade, legend_order)),
                                              "WindOn" = "Onshore wind",
                                              "Waste_heat" = "Waste heat")
  
  my_data_subset = subset(electricity_transfers_simple, Scenario == fnames_split[2, 2] &
                            Model == fnames_split[2, 1] &
                            Site %in% firstup(region_names)[1:3])
  write.csv(my_data_subset, file="01_Inputs_for_plots/Data_Fig2.csv")
  ggplot(data = my_data_subset) +
    geom_col(mapping = aes(x = factor(Site, levels = firstup(region_names)[1:3]), y = .data[["TWh"]], fill = Type)) +
    scale_fill_manual(name = NULL, values = c("#007db3", color_vector[-6])) +
    coord_cartesian(ylim = c(-100, 400), expand = FALSE) +
    facet_grid(. ~scenario.year, switch = "x") +
    ylab("Electricity generation (TWh)") +
    xlab("Year") +
    ggtitle("Electricity generation") +
    theme(strip.placement = "outside",
          panel.border = element_blank(), 
          axis.text.x = element_text(angle = 90, vjust = 0.4),
          axis.title.x=element_blank(),
          legend.box.margin=margin(-10,-10,-10,-10),
          ) +
    guides(fill=guide_legend(nrow=3))
  ggsave(filename = paste("Fig2.jpg", sep=""),
         path = paste(figPath, "02_Outputs", sep = "/"),
         height = 4, width = 5, dpi = 300)
  
  ## Fig S7 ####
  electricity_generation_thai <- subset(electricity_generation_regional, Site=="Thailand" & Scenario=="Typical" & scenario.year != "2016")
  electricity_generation_thai$Type <- recode(electricity_generation_thai$Type, "Onshore wind" = "WindOn", "Waste heat" = "Waste_heat")
  validation_thai <- read_csv(paste(root_folder, "01_Inputs_for_plots", "Data_SI_Fig7.csv", sep="/"))
  validation_thai$Type <- recode(validation_thai$Type, "Energy conservation" = "Efficiency")
  my_dataset <- rbind(electricity_generation_thai, validation_thai) %>%
                spread(Type, TWh)
  my_dataset[is.na(my_dataset)] <- 0
  my_dataset <- group_by(my_dataset, scenario.year, Model, Scenario, Site) %>%
                summarise(Renewables = sum(Solar, WindOn, Bioenergy, Renewables, Waste_heat),
                          Coal = sum(Coal),
                          Gas = sum(Gas),
                          Hydro = sum(Hydro),
                          Efficiency = sum(Efficiency),
                          Cambodia = sum(Cambodia),
                          Oil = sum(Oil), 
                          Laos = sum(Laos),
                          Myanmar = sum(Myanmar)) %>%
                gather(legend_validation, key="Type", value= "TWh")
  my_dataset$Type <- my_dataset$Type %>% factor(levels = legend_validation)

  
  my_dataset$TWh[my_dataset$TWh<0] <- 0
  ggplot(data = my_dataset) +
    geom_bar(mapping = aes(x = Model, y = my_dataset[["TWh"]], fill = Type),
             position = "fill", 
             width = 0.8, 
             stat = "identity") +
    scale_fill_manual(name = NULL, values = color_vector_validation) +
    scale_y_continuous(labels=scales::percent) +
    coord_cartesian(expand = FALSE) +
    facet_grid(. ~scenario.year, switch = "x") +
    ylab("Electricity generation (TWh)") +
    xlab("Year") +
    ggtitle("Validation of electricity generation") +
    theme(strip.placement = "outside",
          panel.border = element_blank(), 
          axis.text.x = element_text(angle = 90, vjust = 0.4),
          axis.title.x=element_blank(),
          ) +
    guides(fill=guide_legend(nrow=3))
  ggsave(filename = paste("SI_Fig7.jpg", sep=""),
         path = paste(figPath, "02_Outputs", sep = "/"),
         height = 4, width = 5, dpi = 300)
  
  ## Fig S6 ####
  electricity_generation_2016 <- subset(electricity_generation_regional, Site %in% firstup(region_names)[1:3] & Scenario=="Typical" & scenario.year == "2016")
  electricity_generation_2016$Type <- recode(electricity_generation_2016$Type, "Onshore wind" = "WindOn", "Waste heat" = "Waste_heat")
  validation_2016 <- read_csv(paste(root_folder, "01_Inputs_for_plots", "Data_SI_Fig6.csv", sep="/"))
  my_dataset <- rbind(electricity_generation_2016, validation_2016) %>%
    spread(Type, TWh)
  my_dataset[is.na(my_dataset)] <- 0
  my_dataset <- group_by(my_dataset, scenario.year, Model, Scenario, Site) %>%
    summarise(Renewables = sum(Bioenergy, Solar, WindOn, Renewables, Waste_heat),
              Coal = sum(Coal),
              Gas = sum(Gas),
              Hydro = sum(Hydro),
              Cambodia = sum(Cambodia),
              Thailand = sum(Thailand),
              Vietnam = sum(Vietnam),
              China = sum(China),
              Other = sum(Oil), 
              Laos = sum(Laos),
              Malaysia = sum(Malaysia)
              ) %>%
    gather(legend_validation2, key="Type", value= "TWh")
  my_dataset$Type <- my_dataset$Type %>% factor(levels = legend_validation2)
  
  ggplot(data = my_dataset) +
    geom_col(mapping = aes(x = factor(Site, levels = firstup(region_names)[1:3]), y = my_dataset[["TWh"]], fill = Type),
             width = 0.8) +
    scale_fill_manual(name = NULL, values = color_vector_validation2) +
    coord_cartesian(ylim = c(-50, 200), expand = FALSE) +
    facet_grid(. ~Model, switch = "x") +
    ylab("Electricity generation (TWh)") +
    xlab("Year") +
    ggtitle("Validation of electricity generation in 2016") +
    theme(strip.placement = "outside",
          panel.border = element_blank(), 
          axis.text.x = element_text(angle = 90, vjust = 0.4),
          axis.title.x=element_blank(),
          ) +
    guides(fill=guide_legend(nrow=4))
  ggsave(filename = paste("SI_Fig6.jpg", sep=""),
         path = paste(figPath, "02_Outputs", sep = "/"),
         height = 4, width = 3, dpi = 300)
}

## Fig S5 ####

if (aux=="focus on energy"){
  
  df <- df_Installed_capacities
  df_all <- do.call("bind_rows", df)
  df_all[is.na(df_all)] <- 0
  df_all <- cbind(select(df_all, !starts_with("Hydro")),
                  transmute(df_all, Hydro = rowSums(select(df_all, starts_with("Hydro")))))
  df_all <- cbind(select(df_all, !starts_with("Solar")),
                  transmute(df_all, Solar = rowSums(select(df_all, starts_with("Solar")))))
  df_all <- cbind(select(df_all, !starts_with("WindOn")),
                  transmute(df_all, WindOn = rowSums(select(df_all, starts_with("WindOn")))))
  
  df <- df_all %>%
    group_by(scenario.year, Model, Scenario) %>%
    summarise(Bioenergy = sum(Bioenergy),
              Coal = sum(Coal),
              Gas = sum(Gas),
              Hydro = sum(Hydro),
              Import = sum(Import),
              Waste_heat = sum(Waste_heat),
              Oil = sum(Oil), 
              Solar = sum(Solar),
              WindOn = sum(WindOn))
  
  installed_capacities_total <- df %>% gather(legend_order,
                                              key = "Type", 
                                              value = "MW")
  
  installed_capacities_total$Type <- recode(installed_capacities_total$Type %>% factor(levels = legend_order),
                                            "WindOn" = "Onshore wind",
                                            "Waste_heat" = "Waste heat")
  installed_capacities_total$GW <- installed_capacities_total$MW / 1000
  installed_capacities_total[abs(installed_capacities_total$GW)<0.1, 'GW'] = 0
  
  for (i in seq_along(model_names)){
    my_data_subset = filter(installed_capacities_total, Model == model_names[i])
    write.csv(my_data_subset, file="01_Inputs_for_plots/Data_SI_Fig5.csv")
    ggplot(data = my_data_subset) +
      geom_col(mapping = aes(x = factor(Scenario, levels = scenario_order), y = .data[["GW"]], fill = Type)) + 
      scale_fill_manual(name = NULL, values = color_vector) +
      coord_cartesian(ylim = c(0, 175), expand = FALSE) +
      facet_grid(. ~scenario.year, switch = "x") +
      ylab("Installed capacities (GW)") +
      xlab("Year") +
      ggtitle("Fig S5") +
      theme(#panel.spacing = unit(0.7, "line"),
        panel.border = element_blank(), 
        strip.placement = "outside",
        axis.text.x = element_text(angle = 90, vjust = 0.4),
        axis.title.x=element_blank(),
        legend.box.margin=margin(-10,-10,-10,-10),
      ) +
      guides(fill=guide_legend(nrow=2))
    ggsave(filename = "SI_Fig5.jpg",
           path = paste(figPath, "02_Outputs", sep = "/"),
           height = 4, width = 5, dpi = 300)
  }
}

## Fig 4a ####

if (aux=="focus on hydro"){
  print("Electricity generation share")
  df <- df_Electricity_generation
  df_all <- do.call("bind_rows", df)
  df_all[is.na(df_all)] <- 0
  df_all <- cbind(select(df_all, !starts_with("Hydro")),
                  transmute(df_all, Hydro = rowSums(select(df_all, starts_with("Hydro")))))
  df_all <- cbind(select(df_all, !starts_with("Solar")),
                  transmute(df_all, Solar = rowSums(select(df_all, starts_with("Solar")))))
  df_all <- cbind(select(df_all, !starts_with("WindOn")),
                  transmute(df_all, WindOn = rowSums(select(df_all, starts_with("WindOn")))))
  df_all <- filter(df_all, Site %in% region_names)
  
  df <- df_all %>%
    group_by(scenario.year, Model, Scenario) %>%
    summarise(Bioenergy = sum(Bioenergy),
              Coal = sum(Coal),
              Gas = sum(Gas),
              Hydro = sum(Hydro),
              Import = sum(Import),
              Waste_heat = sum(Waste_heat),
              Oil = sum(Oil), 
              Solar = sum(Solar),
              WindOn = sum(WindOn))
  
  electricity_generation_total <- df %>% gather(gather_vector, key = "Type", value = "MW")
  electricity_generation_total$TWh <- electricity_generation_total$MW / 10^6
  electricity_generation_total$Type <- recode(electricity_generation_total$Type %>% factor(levels = legend_order),
                                              "WindOn" = "Onshore wind",
                                              "Waste_heat" = "Waste heat")
  
  for (i in seq_along(model_names)){
    my_data_subset = filter(electricity_generation_total, Model == model_names[i])
    write.csv(my_data_subset, file="01_Inputs_for_plots/Data_Fig4a.csv")
    ggplot(data = my_data_subset) +
      geom_col(mapping = aes(x = factor(Scenario, levels = scenario_order), y = .data[["TWh"]], fill = Type)) + 
      scale_fill_manual(name = NULL, values = color_vector) +
      coord_cartesian(ylim = c(0, 450), expand = FALSE) +
      facet_grid(. ~scenario.year, switch = "x") +
      ylab("Electricity generation (TWh)") +
      xlab("Year") +
      ggtitle("Fig4a") +
      theme(#panel.spacing = unit(0.7, "line"),
        panel.border = element_blank(), 
        strip.placement = "outside",
        axis.text.x = element_text(angle = 90, vjust = 0.4),
        axis.title.x=element_blank(),
        legend.box.margin=margin(-10,-10,-10,-10),
      ) +
      guides(fill=guide_legend(nrow=2))
    ggsave(filename = "Fig4a.jpg",
           path = paste(figPath, "02_Outputs", sep = "/"),
           height = 4, width = 5, dpi = 300)
  }
}


## Fig 4b ####

if (aux=="focus on hydro"){
  print("Electricity costs")
  df <- df_System_costs
  df_all <- do.call("rbind", df)
  df_all <- filter(df_all, Site %in% region_names)
  total_costs <- df_all[c("scenario.year", "Fix.costs", "Variable.costs", "Fuel.costs", "Absolute.inv.costs", "Annualized.inv.costs..incl..past.", "Model", "Scenario")] %>% 
    gather(c("Fix.costs", "Variable.costs", "Fuel.costs", "Absolute.inv.costs", "Annualized.inv.costs..incl..past."), key = "Type", value = "Costs")
  total_costs$Costs <- total_costs$Costs / 10^9
  total_costs <- total_costs %>%
    group_by(scenario.year, Model, Scenario, Type) %>%
    summarise(Costs = sum(Costs))
  
  elec_demand <- data.frame(read_excel(refPath, sheet = paste0("Electricity"), na = c("","Eps"))) %>%
    filter(Site %in% region_names) %>%
    select(c("Site", "scenario.year", "elec.demand")) %>%
    group_by(scenario.year) %>%
    summarise(elec.demand = sum(elec.demand))
  elec_demand$elec.demand <- as.numeric(elec_demand$elec.demand) / 10^6
  elec_demand$Type <- "Demand"
  elec_demand$Model <- "URBS"
  elec_demand <- do.call("rbind", replicate(length(scenario_names)*5, elec_demand, simplify = FALSE))
  elec_demand$Scenario <- rep(scenario_names, each=nrow(elec_demand)/length(scenario_names))
  elec_demand <- arrange(elec_demand, scenario.year)
  
  total_costs$LCOE <- total_costs$Costs / elec_demand$elec.demand #USD/kWh
  total_costs$Type <- recode(total_costs$Type, "Fuel.costs" = "Fuel costs",
                             "Variable.costs" = "Variable costs",
                             "Annualized.inv.costs..incl..past." = "Investment costs",
                             "Absolute.inv.costs" = "Absolute inv costs",
                             "Fix.costs" = "Fix costs") %>%
    factor(levels = c("Fuel costs", "Variable costs", "Fix costs", "Investment costs", "Absolute inv costs"))
  
  total_costs_annual <- filter(total_costs, Type != "Absolute inv costs", Type != "Investment costs") %>% 
    select(c("scenario.year", "Model", "Scenario", "Costs")) %>%
    group_by(scenario.year, Model, Scenario) %>%
    summarise(Costs = sum(Costs))
  
  total_costs3 <- filter(total_costs, Type == "Absolute inv costs") %>%
    select(c("Model", "Scenario", "Costs")) %>%
    group_by(Model, Scenario) %>%
    summarise(Costs = sum(Costs))
  
  total_costs3["Costs"] <- total_costs3["Costs"] +
    1 * subset(total_costs_annual, scenario.year == 2016)["Costs"] +
    4 * subset(total_costs_annual, scenario.year == 2020)["Costs"] +
    5 * subset(total_costs_annual, scenario.year == 2025)["Costs"] +
    5 * subset(total_costs_annual, scenario.year == 2030)["Costs"] +
    5 * subset(total_costs_annual, scenario.year == 2035)["Costs"] +
    2 * subset(total_costs_annual, scenario.year == 2037)["Costs"]
  
  write.csv(total_costs3, file="01_Inputs_for_plots/Data_Fig4b.csv")
  ggplot(total_costs3, aes(y = factor(Scenario, levels = scenario_order), x = Costs)) +
    geom_bar(stat='identity', width=0.5, color = color_vector[7], fill = color_vector[7]) + 
    coord_cartesian(xlim = c(400, 440), expand = TRUE) +
    xlab("Cumulative system costs 2016-2037 (billion US$)") +
    ggtitle("Fig4b") +
    theme(
      axis.title.y=element_blank(),
      panel.border = element_blank(),
      strip.background = element_blank(),
    ) +
    geom_text(x=435, y="Stop-All", label="+2.4%", family = "serif", size=3) +
    geom_text(x=434.6, y="Stop-Planned", label="+2.3%", family = "serif", size=3) +
    geom_text(x=432, y="Stop-Main", label="+1.7%", family = "serif", size=3) +
  ggsave(filename = "Fig4b.jpg",
         path = paste(figPath, "02_Outputs", sep = "/"),
         height = 2, width = 4, dpi = 300)
}

## Fig 5 ####

if (aux=="focus on hydro"){
  
  df <- data.frame(read.csv(paste(root_folder, "01_Inputs_for_plots", "Data_Fig5.csv", sep = "/", collapse = NULL)))
  df2 <- tidyr::pivot_longer(df, cols=c('RFI', 'RRI'), names_to='variable', values_to="value")
  
  ggplot(data = df2) +
    geom_bar(stat='identity', position='dodge', mapping = aes(x = factor(Scenario, levels = scenario_order), y = value, fill = variable)) +
    scale_fill_manual(name = NULL, values = color_vector[c(5,1)]) +
    facet_grid(. ~Year, switch = "x") +
    scale_y_continuous(name = "RFI and RRI (%)") +
    xlab("Year") +
    ggtitle("Fig5") +
    theme(#panel.spacing = unit(0.7, "line"),
      panel.border = element_blank(), 
      strip.placement = "outside",
      axis.text.x = element_text(angle = 90, vjust = 0.4),
      axis.title.y.right = element_text(angle = 90),
      axis.title.x=element_blank(),
    ) +
    guides(fill=guide_legend(nrow=1))
  ggsave(filename = "Fig5.jpg",
         path = paste(figPath, "02_Outputs", sep = "/"),
         height = 4, width = 5, dpi = 300)
}