import shapefile as shp
import pandas as pd
import geopandas as gpd

ntc = pd.read_excel("../01_Inputs_for_plots/urbs results/focus on hydro/URBS_Reference.xlsx", sheet_name="NTC")
ntc_2016 = ntc[ntc["scenario-year"]==2016][:-7]
ntc_2016 = ntc_2016.set_index(["Site", "scenario-year"]).stack()
ntc_2016 = ntc_2016[ntc_2016>0].droplevel(1)
ntc_2016 = ntc_2016.reset_index().rename({"Site": "Site in", "level_1": "Site out", 0: "Cap"}, axis=1)

ntc_2037 = ntc[ntc["scenario-year"]==2037][:-7]
ntc_2037 = ntc_2037.set_index(["Site", "scenario-year"]).stack()
ntc_2037 = ntc_2037[ntc_2037>0].droplevel(1)
ntc_2037 = ntc_2037.reset_index().rename({"Site": "Site in", "level_1": "Site out", 0: "Cap"}, axis=1)

regions = gpd.read_file("../03_GIS_figures/Fig3/Mekong_KHM_LAO_THA.shp")
regions = regions[["NAME_SHORT", "geometry"]]
regions["Longitude"] = regions.geometry.centroid.x
regions["Latitude"] = regions.geometry.centroid.y

# Join on index
ntc_2016 = ntc_2016.set_index("Site in").join(regions.rename({"NAME_SHORT": "Site in"}, axis=1).set_index("Site in"), how="inner").reset_index().rename({"Longitude": "Long1", "Latitude":"Lat1"}, axis=1).drop("geometry", axis=1)
ntc_2016 = ntc_2016.set_index("Site out").join(regions.rename({"NAME_SHORT": "Site out"}, axis=1).set_index("Site out"), how="inner").reset_index().rename({"Longitude": "Long2", "Latitude":"Lat2"}, axis=1).drop("geometry", axis=1)
ntc_2016 = ntc_2016.set_index(["Site in", "Site out"])
ntc_2037 = ntc_2037.set_index("Site in").join(regions.rename({"NAME_SHORT": "Site in"}, axis=1).set_index("Site in"), how="inner").reset_index().rename({"Longitude": "Long1", "Latitude":"Lat1"}, axis=1).drop("geometry", axis=1)
ntc_2037 = ntc_2037.set_index("Site out").join(regions.rename({"NAME_SHORT": "Site out"}, axis=1).set_index("Site out"), how="inner").reset_index().rename({"Longitude": "Long2", "Latitude":"Lat2"}, axis=1).drop("geometry", axis=1)
ntc_2037 = ntc_2037.set_index(["Site in", "Site out"])

paths = {}
paths["grid_2016"] = "../03_GIS_figures/Fig3/grid_2016.shp"
paths["grid_2037"] = "../03_GIS_figures/Fig3/grid_2037.shp"
paths["grid_delta"] = "../03_GIS_figures/Fig3/grid_delta.shp"

# Writing to shapefile
with shp.Writer(paths["grid_2016"], shapeType=3) as w:
    w.autoBalance = 1
    w.field("IN", "C", 5, 0)
    w.field("OUT", "C", 5, 0)
    w.field("Cap_MVA", "N", 8, 2)

    for sin, sout in ntc_2016.index:
        if ntc_2016.loc[(sin, sout), "Cap"] == ntc_2016.loc[(sout, sin), "Cap"]:
            ntc_2016.loc[(sout, sin), "Cap"] = 0
        if ntc_2016.loc[(sin, sout), "Cap"] == 0:
            continue
        w.line([[ntc_2016.loc[(sin, sout), ["Long1", "Lat1"]].astype(float), ntc_2016.loc[(sin, sout), ["Long2", "Lat2"]].astype(float)]])
        w.record(sin, sout, ntc_2016.loc[(sin, sout), "Cap"].astype(float))

with shp.Writer(paths["grid_2037"], shapeType=3) as w:
    w.autoBalance = 1
    w.field("IN", "C", 5, 0)
    w.field("OUT", "C", 5, 0)
    w.field("Cap_MVA", "N", 8, 2)

    for sin, sout in ntc_2037.index:
        if ntc_2037.loc[(sin, sout), "Cap"] == ntc_2037.loc[(sout, sin), "Cap"]:
            ntc_2037.loc[(sout, sin), "Cap"] = 0
        if ntc_2037.loc[(sin, sout), "Cap"] == 0:
            continue
        w.line([[ntc_2037.loc[(sin, sout), ["Long1", "Lat1"]].astype(float), ntc_2037.loc[(sin, sout), ["Long2", "Lat2"]].astype(float)]])
        w.record(sin, sout, ntc_2037.loc[(sin, sout), "Cap"].astype(float))
        
with shp.Writer(paths["grid_delta"], shapeType=3) as w:
    w.autoBalance = 1
    w.field("IN", "C", 5, 0)
    w.field("OUT", "C", 5, 0)
    w.field("Cap_MVA", "N", 8, 2)

    for sin, sout in ntc_2037.index:
        if ntc_2037.loc[(sin, sout), "Cap"] == 0:
            continue
        try:
            ntc_2037.loc[(sin, sout), "Cap"] = ntc_2037.loc[(sin, sout), "Cap"] - ntc_2016.loc[(sin, sout), "Cap"]
        except:
            pass
        
        w.line([[ntc_2037.loc[(sin, sout), ["Long1", "Lat1"]].astype(float), ntc_2037.loc[(sin, sout), ["Long2", "Lat2"]].astype(float)]])
        w.record(sin, sout, ntc_2037.loc[(sin, sout), "Cap"].astype(float))