# To read xlsx files
library(tidyverse)
library(readxl)
library(dplyr)

## User preferences ####

gather_vector <- c('Bioenergy','Coal','Gas',  'Hydro', 'Import', 'Oil',
                   'Solar', 'Waste_heat', 'WindOn')

standard <- theme_set(theme_light(base_size = 10, base_family = 'serif'))

theme_replace(legend.position = "top",
              legend.key.size = unit(0.7, "lines"),
              panel.spacing = unit(1, "line"),
              strip.background = element_rect(fill = "grey30", color = NA)
)

color_vector <- c("#FACC2E", #Solar
                  "#884ddb", #WindOn
                  "#7ee0e7", #Hydro
                  "#04B404", #Bioenergy
                  "#B45F04", #Gas
                  "#8B0004", #Import
                  "#232323", #Coal 
                  "#ff0000", #Waste_heat
                  "#83acaf") #Oil

color_vector_countries <- c("#b30000", # Cambodia
                            "#e2bdff", # China
                            "#007db3", # Laos
                            "#ec83eb", # Malaysia
                            "#19f0d4", # Myanmar
                            "#00947b", # Thailand
                            "#ff890a" # Vietnam
                            )
                            
color_vector_validation <- c("#e2fa2e", #Renewables
                             "#7ee0e7", #Hydro
                             "#B45F04", #Gas
                             "#232323", #Coal
                             "#83acaf", #Oil
                             "#ff890a", #Energy conservation
                             "#b30000", # Cambodia
                             "#007db3", # Laos
                             "#19f0d4" # Myanmar
)

color_vector_validation2 <- c("#e2fa2e", #Renewables
                             "#7ee0e7", #Hydro
                             "#B45F04", #Gas
                             "#232323", #Coal
                             "#83acaf", #Other
                             "#b30000", # Cambodia
                             "#e2bdff", # China
                             "#007db3", # Laos
                             "#ec83eb", # Malaysia
                             "#00947b", # Thailand
                             "#ff890a" # Vietnam
)

legend_validation <- c("Renewables",
                       "Hydro", "Gas", "Coal", "Oil", "Efficiency",
                       "Cambodia", "Laos", "Myanmar")

legend_validation2 <- c("Renewables",
                       "Hydro", "Gas", "Coal", "Other", "Cambodia",
                       "China", "Laos", "Malaysia", "Thailand", "Vietnam")

color_vector_emissions <- c("#232323", #Coal 
                            "#DBA901", #Gas
                            "#83acaf")  #Oil

legend_order <- c("Solar", "WindOn", "Hydro", "Bioenergy", 
                  "Gas", "Import",
                  "Coal", "Waste_heat", "Oil")

legend_trade <- c("Cambodia",
                  "China",
                  "Laos",
                  "Malaysia",
                  "Myanmar",
                  "Thailand",
                  "Vietnam")

region_names <- c("thailand", "laos", "cambodia", "myanmar", "malaysia", 
                  "vietnam", "china")

clean_filter <- c("Installed_capacities", "Electricity_generation",# "Emissions_by_fuel",
                  "Electricity", "Total_system_costs")

## Baseline Data Setup ####

root_folder <- getwd()
folders <- list.dirs(path = paste(root_folder, "01_Inputs_for_plots", "urbs results", sep="/"),
					full.names = TRUE, recursive = FALSE)

refFile <- "/focus on hydro/URBS_Reference.xlsx"
refPath <- paste(root_folder, "01_Inputs_for_plots", "urbs results", refFile, sep = "/", collapse = NULL)

if (!dir.exists(paste(root_folder, "02_Outputs/", sep = "/"))) {
  dir.create(file.path(root_folder, "02_Outputs/", collapse="/"))
} 

# Get sheets from single file
mysheets = excel_sheets(path = refPath)

## Loop ####

for (folder in folders){
  aux <- str_split(folder, pattern = "/")[[1]]
  aux <- aux[length(aux)]
  scenario_order <- switch(aux,
                           "focus on energy" = c("Dry", "Typical", "Wet"),
                           "focus on hydro" = c("Stop-All", "Stop-Planned", "Stop-Main", "Reference"),
  )
  dPath <- paste(folder, "/", sep="")
  source("09_Codes_to_generate_plots/generate_plots_paper.R")
}
